@@ -1,6 +1,6 @@
 /*
-  $Id: mblem_mod.h 9084 2011-03-23 10:57:04Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/Frog/trunk/include/frog/mblem_mod.h $
+  $Id: mblem_mod.h 13649 2011-12-06 15:14:20Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/mblem_mod.h $
 
   Copyright (c) 2006 - 2011
   Tilburg University
@@ -48,14 +48,14 @@ class Mblem {
  public:
   Mblem();
   ~Mblem();
-  void init( const Configuration& );
-  void Classify( const UnicodeString& );
-  std::string postprocess( const std::string& tag );
+  bool init( const Configuration& );
+  std::string Classify( folia::FoliaElement * );
  private:
   void read_transtable( const std::string& );
   void create_MBlem_defaults();
   bool readsettings( const std::string& dir, const std::string& fname );
   std::string make_instance( const UnicodeString& in );
+  std::string postprocess( folia::FoliaElement * );
   Timbl::TimblAPI *myLex;
   std::string punctuation;
   size_t history;
