@@ -1,6 +1,6 @@
 /*
-  $Id: mwu_chunker_mod.h 9084 2011-03-23 10:57:04Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/Frog/trunk/include/frog/mwu_chunker_mod.h $
+  $Id: mwu_chunker_mod.h 13685 2011-12-13 14:39:33Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/mwu_chunker_mod.h $
 
   Copyright (c) 2006 - 2011
   Tilburg University
@@ -29,56 +29,28 @@
 #ifndef __MWU_CHUNKER__
 #define __MWU_CHUNKER__
 
-
-class complexAna;
+#include "libfolia/folia.h"
 
 class mwuAna {
   friend std::ostream& operator<< (std::ostream&, const mwuAna& );
  public:
-  mwuAna( const std::string&, const std::string&,
-	  const std::string&, const std::string&,
-	  const std::string&, const std::string& );
+  mwuAna( folia::FoliaElement * );
   virtual ~mwuAna() {};
   
-  virtual complexAna *append( const mwuAna * );
-  std::string getTagHead() const {
-    return tagHead;
-  }
-  
-  std::string getTagMods() const;
-  
+  void merge( const mwuAna * );
+
   std::string getWord() const {
     return word;
   }
-  std::string getLemma() const {
-    return lemma;
-  }
-  std::string getMorphemes() const {
-    return morphemes;
-  }
-  std::string getTag() const {
-    return tag;
-  }
-  
-  virtual std::string displayTag( );
+
+  bool isSpec(){ return spec; };
+  void addEntity( folia::FoliaElement * );
   
  protected:
     mwuAna(){};
     std::string word;
-    std::string tag;
-    std::string tagHead;
-    std::string tagMods;
-    std::string lemma;
-    std::string morphemes;
-    std::string CFS;
-    std::string OFS;
-};
-
-class complexAna: public mwuAna {
- public:
-  complexAna( );
-  complexAna *append( const mwuAna * );
-  std::string displayTag( );
+    bool spec;
+    std::vector<folia::FoliaElement *> fwords;
 };
 
 #define mymap2 std::multimap<std::string, std::vector<std::string> >
@@ -90,15 +62,15 @@ class Mwu {
  public:
   ~Mwu();
   void reset();
-  void init( const Configuration& );
-  void Classify( );
-  void add( const std::string&, const std::string&,
-	    const std::string&, const std::string& );
+  bool init( const Configuration& );
+  void Classify( folia::FoliaElement* );
+  void add( folia::FoliaElement * );
   std::vector<mwuAna*>& getAna(){ return mWords; };
   std::string myCFS;
  private:
   bool readsettings( const std::string&, const std::string&);
   bool read_mwus( const std::string& );
+  void Classify();
   int debug;
   std::string mwuFileName;
   std::vector<mwuAna*> mWords;
