@@ -1,6 +1,6 @@
 /*
-  $Id: Frog.cxx 9084 2011-03-23 10:57:04Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/Frog/trunk/src/Frog.cxx $
+  $Id: Frog.cxx 13765 2011-12-21 13:52:37Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog.cxx $
 
   Copyright (c) 2006 - 2011
   Tilburg University
@@ -41,25 +41,26 @@
 #include <vector>
 #include <omp.h>
 
-
-
 #include "timbl/TimblAPI.h"
 #include "mbt/MbtAPI.h"
+#include "timblserver/TimblServerAPI.h"
+
 // individual module headers
 
 #include "config.h"
 #include "frog/Frog.h"
 #include "frog/Configuration.h"
+#include "ucto/tokenize.h"
 #include "frog/mbma_mod.h"
 #include "frog/mblem_mod.h"
 #include "frog/mwu_chunker_mod.h"
-#include "frog/FrogData.h"
+#include "frog/cgn_tagger_mod.h"
 #include "frog/Parser.h"
-#include "ucto/tokenize.h"
-#include "timblserver/SocketBasics.h"
-
+#include "libfolia/document.h"
+#include "libfolia/folia.h"
 
 using namespace std;
+using namespace folia;
 
 Tokenizer::TokenizerClass tokenizer;
 
@@ -70,6 +71,7 @@ string TestFileName;
 string testDirName;
 string tmpDirName;
 string outputFileName;
+string XMLoutFileName;
 string outputDirName;
 set<string> fileNames;
 string ProgName;
@@ -107,8 +109,8 @@ bool keepIntermediateFiles = false;
 Configuration configuration;
 static string configFileName = string(SYSCONF_PATH) + '/' + PACKAGE + '/' + "frog.cfg";
 
-void usage( const string& progname ) {
-  cout << progname << " v." << VERSION << endl << "Options:\n";
+void usage( ) {
+  cout << endl << "Options:\n";
   cout << "\t============= INPUT MODE (mandatory, choose one) ========================\n"
        << "\t -e <encoding>          specify encoding of the input (default UTF-8)\n"
        << "\t -t <testfile>          Run frog on this file\n"
@@ -125,23 +127,34 @@ void usage( const string& progname ) {
        << "\t --outputdir=<outputfile> Output to directory, instead of default stdout\n"
        << "\t --keep-parser-files=[yes|no] keep intermediate parser files, (last sentence only)\n"
        << "\t============= OTHER OPTIONS ============================================\n"
-       << "\t -d <debug level> (for more verbosity)\n";
+       << "\t -h. give some help.\n"
+       << "\t -V or --version . Show version info.\n"
+       << "\t -d <debug level> (for more verbosity)\n"
+       << "\t -Q               Enable quote detection in tokeniser\n";
 
 }
 
 //**** stuff to process commandline options *********************************
 
-static MbtAPI *tagger;
-
 static Mbma myMbma;
 static Mblem myMblem;
 static Mwu myMwu;
 static Parser myParser;
+static MBTagger myCGNTagger;
 
 bool parse_args( TimblOpts& Opts ) {
   string value;
   bool mood;
-   // is a config file specified?
+  if ( Opts.Find('V', value, mood ) ||
+       Opts.Find("version", value, mood ) ){
+    // we alreade did that
+    exit( EXIT_SUCCESS );
+  }
+  if ( Opts.Find ('h', value, mood)) {
+    usage();
+    exit( EXIT_SUCCESS );
+  };
+  // is a config file specified?
   if ( Opts.Find( 'c',  value, mood ) ) {
     configFileName = value;
     Opts.Delete( 'c' );
@@ -250,9 +263,9 @@ bool parse_args( TimblOpts& Opts ) {
     outputFileName = value;
     Opts.Delete('o');
   };
-  if ( Opts.Find ('h', value, mood)) {
-    usage(ProgName);
-    return false;
+  if ( Opts.Find ('X', value, mood)) {
+    XMLoutFileName = value;
+    Opts.Delete('X');
   };
   if ( Opts.Find ('S', value, mood)) {
     doServer = true;
@@ -273,18 +286,25 @@ bool parse_args( TimblOpts& Opts ) {
     return false;
   }
 
-  if ( doTok ) {
-    string rulesName = configuration.lookUp( "rulesFile", "tokenizer" );
-    if ( rulesName.empty() ){
-      *Log(theErrLog) << "no rulesFile found in configuration" << endl;
-      return false;
-    }
-    else {
-      tokenizer.setErrorLog( theErrLog );
-      if ( !tokenizer.init( configuration.configDir(), rulesName ) )
-	return false;
-    }
+  string rulesName = configuration.lookUp( "rulesFile", "tokenizer" );
+  if ( rulesName.empty() ){
+    *Log(theErrLog) << "no rulesFile found in configuration" << endl;
+    return false;
   }
+  else {
+    tokenizer.setErrorLog( theErrLog );
+    if ( !tokenizer.init( rulesName ) )
+      return false;
+  }
+  string debug = configuration.lookUp( "debug", "tokenizer" );
+  if ( debug.empty() )
+    tokenizer.setDebug( tpDebug );
+  else
+    tokenizer.setDebug( Timbl::stringTo<int>(debug) );
+
+  if ( Opts.Find ('Q', value, mood)) {
+      tokenizer.setQuoteDetection(true);
+  };
 
   if ( !doServer && TestFileName.empty() && fileNames.empty() ){
     *Log(theErrLog) << "no frogging without input!" << endl;
@@ -295,41 +315,52 @@ bool parse_args( TimblOpts& Opts ) {
     // we use fork(). omp (GCC version) doesn't do well when omp is used
     // before the fork!
     // see: http://bisqwit.iki.fi/story/howto/openmp/#OpenmpAndFork
-    string init = "-s " + configuration.configDir() + tagset;
-    tagger = new MbtAPI( init, *theErrLog );
-    myMblem.init( configuration );
-    myMbma.init( configuration );
-    if ( doMwu ){
-      myMwu.init( configuration );
-      if ( doParse )
-	myParser.init( configuration );
+    bool stat = myCGNTagger.init( configuration );
+    if ( stat ){
+      bool stat = myMblem.init( configuration );
+      if ( stat ){
+	stat = myMbma.init( configuration );
+	if ( stat ) {
+	  if ( doMwu ){
+	    stat = myMwu.init( configuration );
+	    if ( stat && doParse )
+	      stat = myParser.init( configuration );
+	  }
+	  else {
+	    if ( doParse )
+	      *Log(theErrLog) << " Parser disabled, because MWU is deselected" << endl;
+	    doParse = false;;
+	  }
+	}
+      }
     }
-    else {
-      if ( doParse )
-	*Log(theErrLog) << " Parser disabled, because MWU is deselected" << endl;
-      doParse = false;;
+    if ( !stat ){
+      *Log(theErrLog) << "Initialization failed." << endl;
+      return false;
     }
   }
-  else if ( true ) {
+  else {
+    bool lemStat = true;
+    bool mwuStat = true;
+    bool mbaStat = true;
+    bool parStat = true;
+    bool tagStat = true;
 #pragma omp parallel sections
     {
 #pragma omp section
-      myMblem.init( configuration );
+      lemStat = myMblem.init( configuration );
 #pragma omp section
-      myMbma.init( configuration );
+      mbaStat = myMbma.init( configuration );
 #pragma omp section 
-      {
-	string init = "-s " + configuration.configDir() + tagset;
-	tagger = new MbtAPI( init, *theErrLog );
-      }
+      tagStat = myCGNTagger.init( configuration );
 #pragma omp section
       {
 	if ( doMwu ){
-	  myMwu.init( configuration );
-	  if ( doParse ){
+	  mwuStat = myMwu.init( configuration );
+	  if ( mwuStat && doParse ){
 	    Common::Timer initTimer;
 	    initTimer.start();
-	    myParser.init( configuration );
+	    parStat = myParser.init( configuration );
 	    initTimer.stop();
 	    *Log(theErrLog) << "init Parse took: " << initTimer << endl;
 	  }
@@ -337,172 +368,80 @@ bool parse_args( TimblOpts& Opts ) {
 	else {
 	  if ( doParse )
 	    *Log(theErrLog) << " Parser disabled, because MWU is deselected" << endl;
-	  doParse = false;;
+	  doParse = false;
 	}
       }
+    }   // end omp parallel sections
+    if ( ! ( tagStat && lemStat && mbaStat && mwuStat && parStat ) ){
+      *Log(theErrLog) << "Initialization failed: ";
+      if ( ! ( tagStat ) ){
+	*Log(theErrLog) << "[tagger] ";
+      }	
+      if ( ! ( lemStat ) ){
+	*Log(theErrLog) << "[lemmatizer] ";
+      }	
+      if ( ! ( mbaStat ) ){
+	*Log(theErrLog) << "[morphology] ";
+      }	
+      if ( ! ( mwuStat ) ){
+	*Log(theErrLog) << "[multiword unit] ";
+      }	
+      if ( ! ( parStat ) ){
+	*Log(theErrLog) << "[parser] ";
+      }
+      *Log(theErrLog) << endl;
+      return false;
     }
-    // end omp parallel sections
   }
   *Log(theErrLog) << "Initialization done." << endl;
   return true;
 }
 
-bool splitOneWT( const string& in, string& word, string& tag ){
-  bool isKnown = true;
-  //     split word and tag, and store num of slashes
-  if (tpDebug)
-    cout << "split Classify starting with " << in << endl;
-  size_t pos = in.rfind("//");
-  if ( pos != string::npos ) {
-    // double slash: lets's hope is is an unknown word
-    if ( pos == 0 ){
-      // but this is definitely something like //LET() 
-      word = "/";
-      tag = in.substr(pos+2);
-    }
-    else {
-      word = in.substr( 0, pos );
-      tag = in.substr( pos+2 );
-    }
-    isKnown = false;
-  } 
-  else {
-    pos = in.rfind("/");
-    if ( pos != string::npos ) {
-      word = in.substr( 0, pos );
-      tag = in.substr( pos+1 );
-    }
-    else {
-      *Log(theErrLog) << "no word/tag pair in this line: " << in << endl;
-      word = "oops";
-      tag = "ai";
-    }
-  }
-  if ( tpDebug){
-    if ( isKnown )
-      cout << "known";
-    else
-      cout << "unknown";
-    cout << " word: " << word << "\ttag: " << tag << endl;
-  }
-  return isKnown;
-}
-
-int splitWT( const string& tagged, const vector<string>& words,
-	     vector<string>& tags, vector<bool>& known ){
-  vector<string> tagwords;
-  tags.clear();
-  known.clear();
-  size_t num_words = split_at( tagged, tagwords, sep );
-  num_words--; // the last "word" is <utt> which gets added by the tagger
-  for( size_t i = 0; i < num_words; ++i ) {
-    string word, tag;
-    bool isKnown = splitOneWT( tagwords[i], word, tag );
-    if ( word != words[i] ){
-      *Log(theErrLog) << "tagger out of sync: word[" << i << "] ≠ tagword["
-		      << i << "] " << word << " vs. " 
-		      << words[i] << endl;
-      exit(1);
-    }
-    tags.push_back( tag );
-    known.push_back( isKnown );
-  }
-  return num_words;
-}
-
-UnicodeString decap( const UnicodeString &W ) {
-  // decapitalize the whole word 
-  UnicodeString w = W;
-  if (tpDebug)
-    cout << "Decapping " << w;
-  for ( int i = 0; i < w.length(); ++i )
-    w.replace( i, 1, u_tolower( w[i] ) );
-  if (tpDebug)
-    cout << " to : " << w << endl;
-  return w;
-}
-
-vector<FrogData *> TestSentence( const string& sentence,
-				 const string& tmpDir,
-				 TimerBlock& timers ){
-  vector<FrogData *> solutions;
-
-  if ( !sentence.empty() ) {
+void TestSentence( FoliaElement* sent,
+		   const string& tmpDir,
+		   TimerBlock& timers ){
+  vector<Word*> swords = sent->words();
+  if ( !swords.empty() ) {
     if (tpDebug) {
-      // don't mangle debug output, so run 1 thread
+      // don't mangle debug output, so run 1 thread then
       omp_set_num_threads( 1 );
     }
-    if (tpDebug) 
-      cout << "in: " << sentence << endl;
-    vector<string> words;
-    int num = split_at( sentence, words, sep );
-    if ( num < 1 ){
-      *theErrLog << "unable to get words from the sentence " << endl;
-      return solutions;
-    }
     timers.tagTimer.start();
-    string tagged = tagger->Tag(sentence);
+    string tagged = myCGNTagger.Classify( sent );
     timers.tagTimer.stop();
-    if (tpDebug) {
-      cout << "sentence: " << sentence
-	   <<"\ntagged: "<< tagged
-	   << endl;
-    }
-    vector<string> tags;
-    vector<bool> known;
-    int num_words = splitWT( tagged, words, tags, known );
-    if (tpDebug) {
-      cout << "#tagged_words: " << num_words << endl;
-      for( int i = 0; i < num_words; i++) 
-	cout   << "\ttagged word[" << i <<"]: " << words[i] << (known[i]?"/":"//")
-	       << tags[i] << endl;
-    }
-    myMwu.reset();
-    for ( int i = 0; i < num_words; ++i ) {
-      //process each word and dump every ana for now
-      UnicodeString uWord = UTF8ToUnicode(words[i] );
-      uWord.toLower();
+    for ( size_t i = 0; i < swords.size(); ++i ) {
       string mbmaLemma;
       string mblemLemma;
-#pragma omp parallel sections firstprivate( uWord )
+#pragma omp parallel sections
       {
 #pragma omp section
 	{
 	  timers.mbmaTimer.start();
 	  if (tpDebug) cout << "Calling mbma..." << endl;
-	  myMbma.Classify( uWord );
-	  if ( tags[i].find( "SPEC(" ) == 0 )
-	    mbmaLemma = "[" + words[i] + "]";
-	  else
-	    mbmaLemma = myMbma.postprocess( uWord, tags[i] );
+	  mbmaLemma = myMbma.Classify( swords[i] );
 	  timers.mbmaTimer.stop();
 	}
 #pragma omp section
 	{
 	  timers.mblemTimer.start();
 	  if (tpDebug) cout << "Calling mblem..." << endl;
-	  myMblem.Classify( uWord );
-	  if ( tags[i].find( "SPEC(" ) == 0 )
-	    mblemLemma = words[i];
-	  else
-	    mblemLemma = myMblem.postprocess( tags[i] ); 
+	  mblemLemma = myMblem.Classify( swords[i] );
 	  timers.mblemTimer.stop();
 	}
       }
       if (tpDebug) {
-	cout   << "tagged word[" << i <<"]: " << words[i] << (known[i]?"/":"//")
-	       << tags[i] << endl;
+	cout   << "tagged word[" << i <<"]: " << swords[i]->str() << " tag "
+	       << swords[i]->pos() << endl;
 	cout << "analysis: " << mblemLemma << " " << mbmaLemma << endl;
       }
-      myMwu.add( words[i], tags[i], mblemLemma, mbmaLemma );  
     } //for int i = 0 to num_words
 
     if ( doMwu ){
-      if ( num_words > 0 ){
+      if ( swords.size() > 0 ){
 	if (tpDebug)
 	  cout << "starting mwu Chunking ... \n";
 	timers.mwuTimer.start();
-	myMwu.Classify( );
+	myMwu.Classify( sent );
 	timers.mwuTimer.stop();
       }
       if (tpDebug) {
@@ -510,13 +449,151 @@ vector<FrogData *> TestSentence( const string& sentence,
 	cout << myMwu << endl;
       }
     }
-    FrogData *pd = new FrogData( myMwu );
     if ( doParse ){  
-      myParser.Parse( pd, tmpDir, timers );
+      myParser.Parse( sent, tmpDir, timers );
     }
-    solutions.push_back( pd );
   }
-  return solutions;
+}
+
+vector<Word*> lookup( Word *word, const vector<Entity*>& entities ){
+  vector<Word*> vec;
+  for ( size_t p=0; p < entities.size(); ++p ){
+    vec = entities[p]->select<Word>();
+    if ( !vec.empty() ){
+      if ( vec[0]->id() == word->id() ) {
+	// cerr << "found " << vec << endl;
+	return vec;
+      }
+    }
+  }
+  vec.clear();
+  vec.push_back( word ); // single unit
+  return vec;
+}
+
+Dependency *lookupDep( const Word *word, 
+		       const vector<Dependency*>&dependencies ){
+  //  cerr << "\nlookup "<< word << " in " << dependencies << endl;
+  for ( size_t i=0; i < dependencies.size(); ++i ){
+    //    cerr << "\nprobeer " << dependencies[i] << endl;
+    try {
+      vector<DependencyDependent*> dv = dependencies[i]->select<DependencyDependent>();
+      if ( !dv.empty() ){
+	vector<Word*> v = dv[0]->select<Word>();
+	for ( size_t j=0; j < v.size(); ++j ){
+	  if ( v[j] == word ){
+	    //	    cerr << "\nfound word " << v[j] << endl;
+	    return dependencies[i];
+	  }
+	}
+      }
+    }
+    catch ( exception& e ){
+      if  (tpDebug > 0) 
+	*Log(theErrLog) << "get Dependency results failed: " 
+			<< e.what() << endl;      
+    }
+  }
+  return 0;
+}
+
+void displayMWU( ostream& os, size_t index, 
+		 const vector<Word*> mwu ){
+  string wrd;
+  string pos;
+  string lemma;
+  string morph;
+  double conf = 1;
+  for ( size_t p=0; p < mwu.size(); ++p ){
+    Word *word = mwu[p];
+    try { 
+      wrd += word->str();
+      PosAnnotation *postag = word->annotation<PosAnnotation>();
+      pos += postag->cls();
+      if ( p < mwu.size() -1 ){
+	wrd += "_";
+	pos += "_";
+      }
+      conf *= postag->confidence();
+    }
+    catch ( exception& e ){
+      if  (tpDebug > 0) 
+	*Log(theErrLog) << "get Postag results failed: " 
+			<< e.what() << endl;            
+    }
+    try { 
+      lemma += word->lemma();
+      if ( p < mwu.size() -1 ){
+	lemma += "_";
+      }
+    }
+    catch ( exception& e ){
+      if  (tpDebug > 0) 
+	*Log(theErrLog) << "get Lemma results failed: " 
+			<< e.what() << endl;      
+    }
+    try { 
+      vector<MorphologyLayer*> ml = word->annotations<MorphologyLayer>();
+      for ( size_t q=0; q < ml.size(); ++q ){
+	vector<Morpheme*> m = ml[q]->select<Morpheme>();
+	for ( size_t t=0; t < m.size(); ++t ){
+	  morph += "[" + UnicodeToUTF8( m[t]->text() ) + "]";
+	}
+	if ( q < ml.size()-1 )
+	  morph += "/";
+      }
+      if ( p < mwu.size() -1 ){
+	morph += "_";
+      }
+    }
+    catch ( exception& e ){
+      if  (tpDebug > 0) 
+	*Log(theErrLog) << "get Morph results failed: " 
+			<< e.what() << endl;      
+    }
+  }
+  os << index << "\t" << wrd << "\t" << lemma << "\t" << morph << "\t" << pos << "\t" << std::fixed << conf;
+}  
+
+ostream &showResults( ostream& os, const FoliaElement* sentence ){
+  vector<Word*> words = sentence->words();
+  vector<Entity*> entities = sentence->select<Entity>();
+  vector<Dependency*> dependencies = sentence->select<Dependency>();
+  size_t index = 1;
+  map<FoliaElement*, int> enumeration;
+  vector<vector<Word*> > mwus;
+  for( size_t i=0; i < words.size(); ++i ){
+    Word *word = words[i];
+    vector<Word*> mwu = lookup( word, entities );
+    for ( size_t j=0; j < mwu.size(); ++j ){
+      enumeration[mwu[j]] = index;
+    }
+    mwus.push_back( mwu );
+    i += mwu.size()-1;
+    ++index;
+  }
+  for( size_t i=0; i < mwus.size(); ++i ){
+    displayMWU( os, i+1, mwus[i] );
+    if ( doParse ){
+      string cls;
+      Dependency *dep = lookupDep( mwus[i][0], dependencies );
+      if ( dep ){
+	vector<DependencyHead*> w = dep->select<DependencyHead>();
+	os << "\t" << enumeration.find(w[0]->index(0))->second << "\t" << dep->cls();
+      }
+      else {
+	os << "\t"<< 0 << "\tROOT";
+      }
+    }
+    else {
+      os << "\t\t";
+    }
+    os << endl;
+    ++index;
+  }
+  if ( words.size() )
+    os << endl;
+  return os;
 }
 
 
@@ -527,61 +604,40 @@ void Test( istream& IN,
 	   Common::Timer &frogTimer,
 	   const string& tmpDir ) {
 
-      bool bos = true; //for pre-tokenised text only
-      string eosmarker = "";
-      tokenizer.setEosMarker( eosmarker );
-      tokenizer.setVerbose( false );
-      tokenizer.setSentenceDetection( true ); //detection of sentences
-      tokenizer.setParagraphDetection( false ); //detection of paragraphs
-      tokenizer.setSentencePerLineInput( doSentencePerLine );
-      if ( !encoding.empty() )
-	tokenizer.setInputEncoding( encoding );
+  string eosmarker = "";
+  tokenizer.setEosMarker( eosmarker );
+  tokenizer.setVerbose( false );
+  tokenizer.setSentenceDetection( true ); //detection of sentences
+  tokenizer.setParagraphDetection( false ); //detection of paragraphs
+  tokenizer.setSentencePerLineInput( doSentencePerLine );
+  if ( !encoding.empty() )
+    tokenizer.setInputEncoding( encoding );
+  
+  tokenizer.setXMLOutput( true, "frog" );
+  string line;  
+  Document doc = tokenizer.tokenize( IN ); 
+  doc.declare( AnnotationType::POS, "mbt-pos");
+  doc.declare( AnnotationType::LEMMA, "mbt-lemma");
 
-      string line;  
-      bool done = false;
-      do {
-	    done = !getline( IN, line );	
-	    if ( doSentencePerLine ){
-	      line += string( " " ) + "<utt>";
-	    }
-	    int numS;
-	    if ((done) || (line.empty())) {
-		tokenizer.signalParagraph();
-		numS = tokenizer.countSentences(true); //count full sentences in token buffer, force buffer to empty!
-	    } else {
-		timers.tokTimer.start();
-		if (doTok) {
-		    tokenizer.tokenizeLine( line ); //tokenize the line, appending to tokens
-		} else {
-		    tokenizer.passthruLine( line, bos ); //add pre-tokenized data without further tokenisation
-		}
-		numS = tokenizer.countSentences(); //count full sentences in token buffer	    
-		timers.tokTimer.stop();
-	    }	
-	    if ( numS > 0 ) { //process sentences 
-	      if  (tpDebug > 0) *Log(theErrLog) << "[tokenize] " << numS << " sentence(s) in buffer, processing..." << endl;
-	      for (int i = 0; i < numS; i++) {
-		/* ******* Begin process sentence  ********** */
-		string sentence = tokenizer.getSentenceString(i);
-		if (tpDebug > 0 ) *Log(theErrLog) << "[DEBUG] SENTENCE: " << sentence << endl;
-		vector<FrogData*> solutions = TestSentence( sentence,  tmpDir, timers ); 
-		//NOTE- full sentences are passed (which may span multiple lines) (MvG)         
-		const size_t solution_size = solutions.size();
-		for ( size_t j = 0; j < solution_size; ++j ) {
-		  showResults( outStream, *solutions[j] ); 
-		  delete solutions[j];
-		}
-	      }
-	      //clear processed sentences from buffer
-	      if  (tpDebug > 0) *Log(theErrLog) << "[tokenize] flushing " << numS << " sentence(s) from buffer..." << endl;
-	      tokenizer.flushSentences(numS);	    
-	    } else {
-	      if  (tpDebug > 0) *Log(theErrLog) << "[tokenize] No sentences yet, reading on..." << endl;
-	    }
-      } while (!done);
+  // Tokenize the whole input into one FoLiA document.
+  // This is nog a good idea on the long term, I think
+
+  vector<Sentence*> sentences = doc.sentences();
+  size_t numS = sentences.size();
+  if ( numS > 0 ) { //process sentences 
+    if  (tpDebug > 0) *Log(theErrLog) << "[tokenize] " << numS << " sentence(s) in buffer, processing..." << endl;
+    for ( size_t i = 0; i < numS; i++) {
+      /* ******* Begin process sentence  ********** */
+      TestSentence( sentences[i],  tmpDir, timers ); 
+      //NOTE- full sentences are passed (which may span multiple lines) (MvG)         
+      showResults( outStream, sentences[i] ); 
+    }
+  } else {
+    if  (tpDebug > 0) *Log(theErrLog) << "[tokenize] No sentences yet, reading on..." << endl;
+  }
   
   frogTimer.stop();  
-
+  
   *Log(theErrLog) << "tokenisation took:" << timers.tokTimer << endl;
   *Log(theErrLog) << "tagging took:     " << timers.tagTimer << endl;
   *Log(theErrLog) << "MBA took:         " << timers.mbmaTimer << endl;
@@ -597,6 +653,10 @@ void Test( istream& IN,
     *Log(theErrLog) << "Parsing (total)   took: " << timers.parseTimer << endl;
   }
   *Log(theErrLog) << "Frogging took:      " << frogTimer << endl;
+  if ( !XMLoutFileName.empty() ){
+    doc.save( XMLoutFileName );
+    *Log(theErrLog) << "resulting FoLiA doc saved in " << XMLoutFileName << endl;
+  }
   return;
 }
 
@@ -612,7 +672,7 @@ void TestFile( const string& infilename, const string& outFileName) {
   if ( !outFileName.empty() ){
     if ( outStream.open( outFileName.c_str() ), outStream.bad() ){
       *Log(theErrLog) << "unable to open outputfile: " << outFileName << endl;
-      exit(1);
+      exit( EXIT_FAILURE );
     }
   }
   TestFileName = infilename; //untokenised!
@@ -637,13 +697,20 @@ void TestServer( Sockets::ServerSocket &conn) {
   Common::Timer frogTimer;
   frogTimer.start();
 
-
-
   try {
     while (true) {
-      string data;
-      if ( !conn.read( data ) )	 //read data from client
-	throw( runtime_error( "read failed" ) );
+      string data = "";      
+      while (true) {
+	string tmpdata;
+	if ( !conn.read( tmpdata ) )	 //read data from client
+	    throw( runtime_error( "read failed" ) );
+	
+	if (data.length() < 2048) { /* Todo: get TCP_BUFFER_SIZE dynamically from TimblServer and OS */
+	    data = data + tmpdata;
+	    break;
+	}
+      }
+          	
       if (tpDebug)
 	std::cerr << "Received: [" << data << "]" << "\n";
       
@@ -668,21 +735,16 @@ void TestServer( Sockets::ServerSocket &conn) {
 
 int main(int argc, char *argv[]) {
   std::ios_base::sync_with_stdio(false);
-  cerr << "frog " << VERSION << "(c) ILK 1998 - 2011" << endl;
+  cerr << "frog " << VERSION << " (c) ILK 1998 - 2011" << endl;
   cerr << "Induction of Linguistic Knowledge Research Group, Tilburg University" << endl;
-  if (argc < 2) {
-    usage(argv[0]);
-    exit(0);
-  }
   ProgName = argv[0];
-  
+  cerr << "based on [" << Tokenizer::VersionName() << ", "
+       << folia::VersionName() << ", "
+       << Timbl::VersionName() << ", "
+       << TimblServer::VersionName() << ", "
+       << Tagger::VersionName() << "]" << endl;
   try {
     TimblOpts Opts(argc, argv);
-    bool dummy;
-    string val;
-    if ( Opts.Find('v', val, dummy ) ){
-      exit(0);
-    }
     if ( parse_args(Opts) ){
       //gets a settingsfile for each component, 
       //and starts init for that mod
@@ -747,11 +809,15 @@ int main(int argc, char *argv[]) {
 	TestFile( TestFileName, outputFileName );
       }
     }
+    else {
+      return EXIT_FAILURE;
+    }
   }
   catch ( const exception& e ){
     *Log(theErrLog) << "fatal error: " << e.what() << endl;
+    return EXIT_FAILURE;
   }
-  delete tagger;
+  //  delete tagger;
   
-  return 0;
+  return EXIT_SUCCESS;
 }
