@@ -1,6 +1,6 @@
 /*
-  $Id: Parser.cxx 9084 2011-03-23 10:57:04Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/Frog/trunk/src/Parser.cxx $
+  $Id: Parser.cxx 13765 2011-12-21 13:52:37Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Parser.cxx $
 
   Copyright (c) 2006 - 2011
   Tilburg University
@@ -37,39 +37,56 @@
 
 #include "config.h"
 #include "timbl/TimblAPI.h"
+#include "libfolia/folia.h"
 #include "frog/Frog.h"
 #include "frog/Configuration.h"
-#include "frog/mwu_chunker_mod.h"
-#include "frog/FrogData.h"
 #include "frog/Parser.h"
 
 using namespace std;
-using namespace Timbl;
+using namespace folia;
 
 PythonInterface::PythonInterface( ) {
   Py_OptimizeFlag = 1; // enable optimisation (-O) mode
   Py_Initialize();
-  string newpath = Py_GetPath();
-  newpath += string(":") + PYTHONDIR;
-  PySys_SetPath( (char*)newpath.c_str() );
-  try {
-    PyObject *im = PyImport_ImportModule( "csidp" );
-    if ( im ){
-      module.assign( im );
-      PyObject *mf = PyObject_GetAttrString(module, "main");
-      if ( mf )
-	mainFunction.assign( mf );
-      else {
-	PyErr_Print();
-	exit(1);
-      }
+
+  PyObject* ourpath = PyString_FromString( PYTHONDIR );
+  if ( ourpath != 0 ){
+    PyObject *sys_path = PySys_GetObject( (char*)("path"));
+    if (sys_path != NULL ){
+      PyList_Append(sys_path, ourpath);
     }
+    else
+      sys_path = ourpath;
+  }
+  PyObject *im = 0;
+  try {
+    im = PyImport_ImportModule( "frog.csidp" );
+  }
+  catch( exception const & ){
+    PyErr_Print();
+    im = 0;
+  }
+  PyErr_Clear();
+  if ( !im ){
+    try {
+      im = PyImport_ImportModule( "csidp" );
+    }
+    catch( exception const & ){
+      PyErr_Print();
+      exit(1);
+    }
+  }
+  if ( im ){
+    module.assign( im );
+    PyObject *mf = PyObject_GetAttrString(module, "main");
+    if ( mf )
+      mainFunction.assign( mf );
     else {
       PyErr_Print();
       exit(1);
     }
   }
-  catch( exception const & ){
+  else {
     PyErr_Print();
     exit(1);
   }
@@ -102,9 +119,44 @@ Common::Timer pairsTimer;
 Common::Timer dirTimer;
 Common::Timer csiTimer;
 
-void Parser::createParserFile( ostream& os ){
-  for( size_t i = 0; i < mwu_data->size(); ++i )
-    os << i+1 << (*mwu_data)[i]->displayTag( ) << endl;
+struct parseData {
+  void clear() { words.clear(); heads.clear(); mods.clear(); mwus.clear(); }
+  vector<string> words;
+  vector<string> heads;
+  vector<string> mods;
+  vector<vector<Word*> > mwus;
+};
+
+ostream& operator<<( ostream& os, const parseData& pd ){
+  os << "pd words " << pd.words << endl;
+  os << "pd heads " << pd.heads << endl;
+  os << "pd mods " << pd.mods << endl;
+  os << "pd mwus ";
+  for ( size_t i=0; i < pd.mwus.size(); ++i ){
+    for ( size_t j=0; j < pd.mwus[i].size(); ++j )
+      os << pd.mwus[i][j]->id();
+    os << endl;
+  }
+  return os;
+}
+  
+void Parser::createParserFile( const parseData& pd ){
+  const vector<string>& words = pd.words;
+  const vector<string>& heads = pd.heads;
+  const vector<string>& mods = pd.mods;
+
+  ofstream anaFile( fileName.c_str() );
+  if ( anaFile ){
+    for( size_t i = 0; i < words.size(); ++i ){
+      anaFile << i+1 << "\t" << words[i] << "\t" << "*" << "\t" << heads[i] 
+	      << "\t" << heads[i] << "\t" << mods[i] << "\t"<< "0" 
+	      << "\t" << "_" << "\t" << "_" << "\t" << "_" << endl;
+    }
+  }
+  else {
+    cerr << "unable to create a parser file " << fileName << endl;
+    exit( EXIT_FAILURE );
+  }
 }
 
 bool Parser::init( const Configuration& configuration ){
@@ -124,8 +176,8 @@ bool Parser::init( const Configuration& configuration ){
   string cDir = configuration.configDir();
   string att = configuration.lookUp( "maxDepSpan", "parser" );
   if ( !att.empty() ){
-    size_t gs;
-    if ( stringTo<size_t>( att, gs, 0, 50 ) ){
+    size_t gs = stringTo<size_t>( att );
+    if ( gs < 50 ){
       maxDepSpanS = att;
       maxDepSpan = gs;
     }
@@ -183,7 +235,7 @@ bool Parser::init( const Configuration& configuration ){
     return false;
   
   bool happy = true;
-  pairs = new TimblAPI( pairsOptions );
+  pairs = new Timbl::TimblAPI( pairsOptions );
   if ( pairs->Valid() ){
     *Log(parseLog) << "reading " <<  pairsFileName << endl;
     happy = pairs->GetInstanceBase( pairsFileName );
@@ -191,7 +243,7 @@ bool Parser::init( const Configuration& configuration ){
   else
     *Log(parseLog) << "creating Timbl for pairs failed:" << pairsOptions << endl;
   if ( happy ){
-    dir = new TimblAPI( dirOptions );
+    dir = new Timbl::TimblAPI( dirOptions );
     if ( dir->Valid() ){
       *Log(parseLog) << "reading " <<  dirFileName << endl;
       happy = dir->GetInstanceBase( dirFileName );
@@ -199,7 +251,7 @@ bool Parser::init( const Configuration& configuration ){
     else
       *Log(parseLog) << "creating Timbl for dir failed:" << dirOptions << endl;
     if ( happy ){
-      rels = new TimblAPI( relsOptions );
+      rels = new Timbl::TimblAPI( relsOptions );
       if ( rels->Valid() ){
 	*Log(parseLog) << "reading " <<  relsFileName << endl;
 	happy = rels->GetInstanceBase( relsFileName );
@@ -219,21 +271,40 @@ Parser::~Parser(){
   delete PI;
 }
 
-void Parser::createPairs(){
+static vector<Word *> lookup( Word *word, 
+			      const vector<Entity*>& entities ){
+  vector<Word*> vec;
+  for ( size_t p=0; p < entities.size(); ++p ){
+    vec = entities[p]->select<Word>();
+    if ( !vec.empty() ){
+      if ( vec[0]->id() == word->id() ) {
+	// cerr << "found " << vec << endl;
+	return vec;
+      }
+    }
+  }
+  vec.clear();
+  return vec;
+}
+
+void Parser::createPairs( const parseData& pd ){
+  const vector<string>& words = pd.words;
+  const vector<string>& heads = pd.heads;
+  const vector<string>& mods = pd.mods;
   string pFile = fileName + ".pairs.inst";
   remove( pFile.c_str() );
   ofstream ps( pFile.c_str() );
   if ( ps ){
-    if ( mwu_data->size() == 1 ){
-      ps << "__ " << (*mwu_data)[0]->getWord() << " __"
-	 << " ROOT ROOT ROOT __ " << (*mwu_data)[0]->getTagHead() 
+    if ( words.size() == 1 ){
+      ps << "__ " << words[0] << " __"
+	 << " ROOT ROOT ROOT __ " << heads[0]
 	 << " __ ROOT ROOT ROOT "
-	 << (*mwu_data)[0]->getTagHead() << "^ROOT ROOT ROOT ROOT^" 
-	 << (*mwu_data)[0]->getTagMods() 
+	 << words[0] << "^ROOT ROOT ROOT ROOT^" 
+	 << heads[0]
 	 << " _" << endl;
     }
     else {
-      for ( size_t i=0 ; i < mwu_data->size(); ++i ){
+      for ( size_t i=0 ; i < words.size(); ++i ){
 	string word_1, word0, word1;
 	string tag_1, tag0, tag1;
 	string mods0;
@@ -242,19 +313,19 @@ void Parser::createPairs(){
 	  tag_1 = "__";
 	}
 	else {
-	  word_1 = (*mwu_data)[i-1]->getWord();
-	  tag_1 = (*mwu_data)[i-1]->getTagHead();
+	  word_1 = words[i-1];
+	  tag_1 = heads[i-1];
 	}
-	word0 = (*mwu_data)[i]->getWord();
-	tag0 = (*mwu_data)[i]->getTagHead();
-	mods0 = (*mwu_data)[i]->getTagMods();
-	if ( i == mwu_data->size() - 1 ){
+	word0 = words[i];
+	tag0 = heads[i];
+	mods0 = mods[i];
+	if ( i == words.size() - 1 ){
 	  word1 = "__";
 	  tag1 = "__";
 	}
 	else {
-	  word1 = (*mwu_data)[i+1]->getWord();
-	  tag1 = (*mwu_data)[i+1]->getTagHead();
+	  word1 = words[i+1];
+	  tag1 = heads[i+1];
 	}
 	ps << word_1 << " " << word0 << " " << word1
 	   << " ROOT ROOT ROOT "
@@ -264,7 +335,7 @@ void Parser::createPairs(){
 	   << " _" << endl;
       }
       // 
-      for ( size_t wPos=0; wPos < mwu_data->size(); ++wPos ){
+      for ( size_t wPos=0; wPos < words.size(); ++wPos ){
 	string w_word_1, w_word0, w_word1;
 	string w_tag_1, w_tag0, w_tag1;
 	string w_mods0;
@@ -273,21 +344,21 @@ void Parser::createPairs(){
 	  w_tag_1 = "__";
 	}
 	else {
-	  w_word_1 = (*mwu_data)[wPos-1]->getWord();
-	  w_tag_1 = (*mwu_data)[wPos-1]->getTagHead();
+	  w_word_1 = words[wPos-1];
+	  w_tag_1 = heads[wPos-1];
 	}
-	w_word0 = (*mwu_data)[wPos]->getWord();
-	w_tag0 = (*mwu_data)[wPos]->getTagHead();
-	w_mods0 = (*mwu_data)[wPos]->getTagMods();
-	if ( wPos == mwu_data->size()-1 ){
+	w_word0 = words[wPos];
+	w_tag0 = heads[wPos];
+	w_mods0 = mods[wPos];
+	if ( wPos == words.size()-1 ){
 	  w_word1 = "__";
 	  w_tag1 = "__";
 	}
 	else {
-	  w_word1 = (*mwu_data)[wPos+1]->getWord();
-	  w_tag1 = (*mwu_data)[wPos+1]->getTagHead();
+	  w_word1 = words[wPos+1];
+	  w_tag1 = heads[wPos+1];
 	}
-	for ( size_t pos=0; pos < mwu_data->size(); ++pos ){
+	for ( size_t pos=0; pos < words.size(); ++pos ){
 	  //	  os << wPos << "-" << pos << " ";
 	  if ( pos > wPos + maxDepSpan )
 	    break;
@@ -303,13 +374,13 @@ void Parser::createPairs(){
 	  if ( pos == 0 )
 	    ps << " __";
 	  else
-	    ps << " " << (*mwu_data)[pos-1]->getWord();
-	  if ( pos < mwu_data->size() )
-	    ps << " " << (*mwu_data)[pos]->getWord();
+	    ps << " " << words[pos-1];
+	  if ( pos < words.size() )
+	    ps << " " << words[pos];
 	  else
 	    ps << " __";
-	  if ( pos < mwu_data->size()-1 )
-	    ps << " " << (*mwu_data)[pos+1]->getWord();
+	  if ( pos < words.size()-1 )
+	    ps << " " << words[pos+1];
 	  else
 	    ps << " __";
 	  ps << " " << w_tag_1;
@@ -318,19 +389,19 @@ void Parser::createPairs(){
 	  if ( pos == 0 )
 	    ps << " __";
 	  else
-	    ps << " " << (*mwu_data)[pos-1]->getTagHead();
-	  if ( pos < mwu_data->size() )
-	    ps << " " << (*mwu_data)[pos]->getTagHead();
+	    ps << " " << heads[pos-1];
+	  if ( pos < words.size() )
+	    ps << " " << heads[pos];
 	  else
 	    ps << " __";
-	  if ( pos < mwu_data->size()-1 )
-	    ps << " " << (*mwu_data)[pos+1]->getTagHead();
+	  if ( pos < words.size()-1 )
+	    ps << " " << heads[pos+1];
 	  else
 	    ps << " __";
 	  
 	  ps << " " << w_tag0 << "^";
-	  if ( pos < mwu_data->size() )
-	    ps << (*mwu_data)[pos]->getTagHead();
+	  if ( pos < words.size() )
+	    ps << heads[pos];
 	  else
 	    ps << "__";
 	  
@@ -338,10 +409,10 @@ void Parser::createPairs(){
 	    ps << " LEFT " << wPos - pos;
 	  else
 	    ps << " RIGHT " << pos - wPos;
-	  if ( pos >= mwu_data->size() )
+	  if ( pos >= words.size() )
 	    ps << " __";
 	  else
-	    ps << " " << (*mwu_data)[pos]->getTagMods();
+	    ps << " " << mods[pos];
 	  ps << "^" << w_mods0;
 	  ps << " __" << endl;
 	}
@@ -350,7 +421,11 @@ void Parser::createPairs(){
   }
 }
 
-void Parser::createRelDir(){
+void Parser::createRelDir( const parseData& pd ){
+  const vector<string>& words = pd.words;
+  const vector<string>& heads = pd.heads;
+  const vector<string>& mods = pd.mods;
+
   string word_2, word_1, word0, word1, word2;
   string tag_2, tag_1, tag0, tag1, tag2;
   string mod_2, mod_1, mod0, mod1, mod2;
@@ -361,10 +436,10 @@ void Parser::createRelDir(){
   remove( rFile.c_str() );
   ofstream rs( rFile.c_str() );
   if ( ds && rs ){
-    if ( mwu_data->size() == 1 ){
-      word0 = (*mwu_data)[0]->getWord();
-      tag0 = (*mwu_data)[0]->getTagHead();
-      mod0 = (*mwu_data)[0]->getTagMods();
+    if ( words.size() == 1 ){
+      word0 = words[0];
+      tag0 = heads[0];
+      mod0 = mods[0];
       ds << "__ __";
       ds << " " << word0;
       ds << " __ __ __ __";
@@ -390,13 +465,13 @@ void Parser::createRelDir(){
       rs << " " << tag0 << "^__^__";
       rs << " __" << endl;
     }
-    else if ( mwu_data->size() == 2 ){
-      word0 = (*mwu_data)[0]->getWord();
-      tag0 = (*mwu_data)[0]->getTagHead();
-      mod0 = (*mwu_data)[0]->getTagMods();
-      word1 = (*mwu_data)[1]->getWord();
-      tag1 = (*mwu_data)[1]->getTagHead();
-      mod1 = (*mwu_data)[1]->getTagMods();
+    else if ( words.size() == 2 ){
+      word0 = words[0];
+      tag0 = heads[0];
+      mod0 = mods[0];
+      word1 = words[1];
+      tag1 = heads[1];
+      mod1 = mods[1];
       ds << "__ __";
       ds << " " << word0;
       ds << " " << word1;
@@ -458,16 +533,16 @@ void Parser::createRelDir(){
       rs << " " << tag1 << "^__^__";
       rs << " __" << endl;
     }
-    else if ( mwu_data->size() == 3 ) {
-      word0 = (*mwu_data)[0]->getWord();
-      tag0 = (*mwu_data)[0]->getTagHead();
-      mod0 = (*mwu_data)[0]->getTagMods();
-      word1 = (*mwu_data)[1]->getWord();
-      tag1 = (*mwu_data)[1]->getTagHead();
-      mod1 = (*mwu_data)[1]->getTagMods();
-      word2 = (*mwu_data)[2]->getWord();
-      tag2 = (*mwu_data)[2]->getTagHead();
-      mod2 = (*mwu_data)[2]->getTagMods();
+    else if ( words.size() == 3 ) {
+      word0 = words[0];
+      tag0 = heads[0];
+      mod0 = mods[0];
+      word1 = words[1];
+      tag1 = heads[1];
+      mod1 = mods[1];
+      word2 = words[2];
+      tag2 = heads[2];
+      mod2 = mods[2];
       ds << "__ __";
       ds << " " << word0;
       ds << " " << word1;
@@ -532,7 +607,7 @@ void Parser::createRelDir(){
       rs << " __ __";
       rs << " " << tag0;
       rs << " " << tag1;
-      rs << " " << (*mwu_data)[2]->getTagHead();
+      rs << " " << tag2;
       rs << " __^" << tag0;
       rs << " " << tag0 << "^" << tag1;
       rs << " __^__^" << tag0;
@@ -570,7 +645,7 @@ void Parser::createRelDir(){
       rs << " __" << endl;
     }
     else {
-      for ( size_t i=0 ; i < mwu_data->size(); ++i ){
+      for ( size_t i=0 ; i < words.size(); ++i ){
 	if ( i == 0 ){
 	  word_2 = "__";
 	  tag_2 = "__";
@@ -583,33 +658,33 @@ void Parser::createRelDir(){
 	  word_2 = "__";
 	  tag_2 = "__";
 	  mod_2 = "__";
-	  word_1 = (*mwu_data)[i-1]->getWord();
-	  tag_1 = (*mwu_data)[i-1]->getTagHead();
-	  mod_1 = (*mwu_data)[i-1]->getTagMods();
+	  word_1 = words[i-1];
+	  tag_1 = heads[i-1];
+	  mod_1 = mods[i-1];
 	}
 	else {
-	  word_2 = (*mwu_data)[i-2]->getWord();
-	  tag_2 = (*mwu_data)[i-2]->getTagHead();
-	  mod_2 = (*mwu_data)[i-2]->getTagMods();
-	  word_1 = (*mwu_data)[i-1]->getWord();
-	  tag_1 = (*mwu_data)[i-1]->getTagHead();
-	  mod_1 = (*mwu_data)[i-1]->getTagMods();
+	  word_2 = words[i-2];
+	  tag_2 = heads[i-2];
+	  mod_2 = mods[i-2];
+	  word_1 = words[i-1];
+	  tag_1 = heads[i-1];
+	  mod_1 = mods[i-1];
 	}
-	word0 = (*mwu_data)[i]->getWord();
-	tag0 = (*mwu_data)[i]->getTagHead();
-	mod0 = (*mwu_data)[i]->getTagMods();
-	if ( i < mwu_data->size() - 2 ){
-	  word1 = (*mwu_data)[i+1]->getWord();
-	  tag1 = (*mwu_data)[i+1]->getTagHead();
-	  mod1 = (*mwu_data)[i+1]->getTagMods();
-	  word2 = (*mwu_data)[i+2]->getWord();
-	  tag2 = (*mwu_data)[i+2]->getTagHead();
-	  mod2 = (*mwu_data)[i+2]->getTagMods();
+	word0 = words[i];
+	tag0 = heads[i];
+	mod0 = mods[i];
+	if ( i < words.size() - 2 ){
+	  word1 = words[i+1];
+	  tag1 = heads[i+1];
+	  mod1 = mods[i+1];
+	  word2 = words[i+2];
+	  tag2 = heads[i+2];
+	  mod2 = mods[i+2];
 	}
-	else if ( i == mwu_data->size() - 2 ){
-	  word1 = (*mwu_data)[i+1]->getWord();
-	  tag1 = (*mwu_data)[i+1]->getTagHead();
-	  mod1 = (*mwu_data)[i+1]->getTagMods();
+	else if ( i == words.size() - 2 ){
+	  word1 = words[i+1];
+	  tag1 = heads[i+1];
+	  mod1 = mods[i+1];
 	  word2 = "__";
 	  tag2 = "__";
 	  mod2 = "__";
@@ -665,29 +740,143 @@ void Parser::createRelDir(){
   }
 }
 
-void Parser::prepareParse( ) {
+void Parser::prepareParse( FoliaElement *sent, parseData& pd ) {
+
+  pd.clear();
+  vector<Word*> fwords = sent->words();
+  vector<Entity*> entities = sent->select<Entity>();
+  
+  for( size_t i=0; i < fwords.size(); ++i ){
+    Word *word = fwords[i];
+    vector<Word*> mwu = lookup( word, entities );
+    if ( !mwu.empty() ){
+      string word;
+      string head;
+      string mod;
+      for ( size_t p=0; p < mwu.size(); ++p ){
+	word += mwu[p]->str();
+	PosAnnotation *postag = mwu[p]->annotation<PosAnnotation>();
+	head += postag->feat("head");
+	vector<folia::Feature*> feats = postag->select<folia::Feature>();
+	for ( size_t j=0; j < feats.size(); ++j ){
+	  mod += feats[j]->cls();
+	  if ( j < feats.size()-1 )
+	    mod += "|";
+	}
+	if ( p < mwu.size() -1 ){
+	  word += "_";
+	  head += "_";
+	  mod += "_";
+	}
+      }
+      pd.words.push_back( word );
+      pd.heads.push_back( head );
+      pd.mods.push_back( mod );
+      pd.mwus.push_back( mwu );
+      i += mwu.size()-1;
+    }
+    else {
+      pd.words.push_back( word->str() );
+      PosAnnotation *postag = word->annotation<PosAnnotation>();
+      string head = postag->feat("head");
+      pd.heads.push_back( head );
+      string mod;
+      vector<folia::Feature*> feats = postag->select<folia::Feature>();
+      if ( feats.size() == 0 )
+	mod = "__";
+      else {
+	for ( size_t j=0; j < feats.size(); ++j ){
+	  mod += feats[j]->cls();
+	  if ( j < feats.size()-1 )
+	    mod += "|";
+	}
+      }
+      pd.mods.push_back( mod );
+      vector<Word*> vec;
+      vec.push_back(word);
+      pd.mwus.push_back( vec );
+    }
+  }
+  
+  createParserFile( pd );
+
 #pragma omp parallel sections
   {
 #pragma omp section
     {
-      createPairs( );
+      createPairs( pd );
     }
 #pragma omp section
     {
-      createRelDir();
+      createRelDir( pd );
     }
   }
 }
-  
-void Parser::Parse( FrogData *pd, const string& tmpDirName, TimerBlock& timers ){
-  mwu_data = &pd->mwu_data;
+
+ void appendParseResult( FoliaElement *sent, 
+			 parseData& pd,
+			 istream& is ){
+   string line;
+   int cnt=0;
+   vector<int> nums;
+   vector<string> roles;
+   while ( getline( is, line ) ){
+     vector<string> parts;
+     int num = Timbl::split_at( line, parts, " " );
+     if ( num > 7 ){
+       if ( Timbl::stringTo<int>( parts[0] ) != cnt+1 ){
+	 *Log(theErrLog) << "confused! " << endl;
+	 *Log(theErrLog) << "got line '" << line << "'" << endl;
+       }
+       nums.push_back( Timbl::stringTo<int>(parts[6]) );
+       roles.push_back( parts[7] );
+     }
+     ++cnt;
+   }
+   FoliaElement *dl = new DependenciesLayer("");
+#pragma omp critical(foliaupdate)
+   {
+   sent->append( dl );
+   }
+   for ( size_t i=0; i < nums.size(); ++i ){
+     if ( nums[i] != 0 ){
+       FoliaElement *d = new Dependency( "generate-id='" + sent->id() + 
+					    "', class='"+ roles[i] +"'" );
+#pragma omp critical(foliaupdate)
+       {
+       dl->append( d );
+       }
+       FoliaElement *h = new DependencyHead("");
+#pragma omp critical(foliaupdate)
+       {
+	 for ( size_t j=0; j < pd.mwus[nums[i]-1].size(); ++ j ){
+	   h->append( pd.mwus[nums[i]-1][j] );
+	 }
+	 d->append( h );
+       }
+
+#pragma omp critical(foliaupdate)
+       {
+	 h = new DependencyDependent("");
+	 for ( size_t j=0; j < pd.mwus[i].size(); ++ j ){
+	   h->append( pd.mwus[i][j] );
+	 }
+	 d->append( h );
+       }
+     }
+   }
+ }
+ 
+ 
+void Parser::Parse( FoliaElement *sent,
+		    const string& tmpDirName, TimerBlock& timers ){
   fileName = tmpDirName+"csiparser";
   timers.parseTimer.start();
   if ( !isInit ){
     *Log(parseLog) << "Parser is not initialized!" << endl;
     exit(1);
   }
-  if ( mwu_data->empty() ){
+  if ( !sent ){
     *Log(parseLog) << "unable to parse an analisis without words" << endl;
     return;
   }
@@ -698,68 +887,66 @@ void Parser::Parse( FrogData *pd, const string& tmpDirName, TimerBlock& timers )
   string dirOutName = fileName + ".dir.out";
   string relsInName = fileName + ".rels.inst";
   string relsOutName = fileName + ".rels.out";
-  ofstream anaFile( fileName.c_str() );
-  if ( anaFile ){
-    createParserFile( anaFile );
-    remove( resFileName.c_str() );
-    timers.prepareTimer.start();
-    prepareParse( );
-    timers.prepareTimer.stop();
+  remove( resFileName.c_str() );
+  timers.prepareTimer.start();
+  parseData pd;  
+  prepareParse( sent, pd );
+  timers.prepareTimer.stop();
 #pragma omp parallel sections
+  {
+#pragma omp section
     {
-#pragma omp section
-      {
-	remove( pairsOutName.c_str() );
-	timers.pairsTimer.start();
-	pairs->Test( pairsInName, pairsOutName );
-	timers.pairsTimer.stop();
-      }
-#pragma omp section
-      {
-	remove( dirOutName.c_str() );
-	timers.dirTimer.start();
-	dir->Test( dirInName, dirOutName );
-	timers.dirTimer.stop();
-      }
-#pragma omp section
-      {
-	remove( relsOutName.c_str() );
-	timers.relsTimer.start();
-	rels->Test( relsInName, relsOutName );
-	timers.relsTimer.stop();
-      }
-    }
-    timers.csiTimer.start();
-    try {
-      PI->parse( pairsOutName,
-		 relsOutName,
-		 dirOutName,
-		 maxDepSpanS,
-		 fileName,
-		 resFileName );
-      if ( PyErr_Occurred() )
-	PyErr_Print();
-    }
-    catch( exception const & ){
-      PyErr_Print();
-    }
-    timers.csiTimer.stop();
-    ifstream resFile( resFileName.c_str() );
-    if ( resFile ){
-      pd->appendParseResult( resFile );
-    }
-    else
-      *Log(parseLog) << "couldn't open results file: " << resFileName << endl;
-    if ( !keepIntermediateFiles ){
-      remove( fileName.c_str() );
-      remove( resFileName.c_str() );
       remove( pairsOutName.c_str() );
-      remove( dirOutName.c_str() );
-      remove( relsOutName.c_str() );
-      remove( pairsInName.c_str() );
-      remove( dirInName.c_str() );
-      remove( relsInName.c_str() );
+      timers.pairsTimer.start();
+      pairs->Test( pairsInName, pairsOutName );
+      timers.pairsTimer.stop();
     }
+#pragma omp section
+    {
+      remove( dirOutName.c_str() );
+      timers.dirTimer.start();
+      dir->Test( dirInName, dirOutName );
+      timers.dirTimer.stop();
+    }
+#pragma omp section
+    {
+      remove( relsOutName.c_str() );
+      timers.relsTimer.start();
+      rels->Test( relsInName, relsOutName );
+      timers.relsTimer.stop();
+    }
+  }
+  timers.csiTimer.start();
+  try {
+    PI->parse( pairsOutName,
+	       relsOutName,
+	       dirOutName,
+	       maxDepSpanS,
+	       fileName,
+	       resFileName );
+    if ( PyErr_Occurred() )
+      PyErr_Print();
+  }
+  catch( exception const & ){
+    PyErr_Print();
+  }
+  timers.csiTimer.stop();
+  ifstream resFile( resFileName.c_str() );
+  if ( resFile ){
+    appendParseResult( sent, pd, resFile );
+  }
+  else
+    *Log(parseLog) << "couldn't open results file: " << resFileName << endl;
+
+  if ( !keepIntermediateFiles ){
+    remove( fileName.c_str() );
+    remove( resFileName.c_str() );
+    remove( pairsOutName.c_str() );
+    remove( dirOutName.c_str() );
+    remove( relsOutName.c_str() );
+    remove( pairsInName.c_str() );
+    remove( dirInName.c_str() );
+    remove( relsInName.c_str() );
   }
   timers.parseTimer.stop();
 }
