@@ -1,6 +1,6 @@
 /*
-  $Id: mbma_mod.cxx 9084 2011-03-23 10:57:04Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/Frog/trunk/src/mbma_mod.cxx $
+  $Id: mbma_mod.cxx 13763 2011-12-21 12:07:49Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mbma_mod.cxx $
 
   Copyright (c) 2006 - 2011
   Tilburg University
@@ -34,12 +34,12 @@
 #include <timbl/TimblAPI.h>
 
 #include "frog/Frog.h"
-#include "ucto/unicode.h"
 #include "frog/Configuration.h"
+#include "libfolia/folia.h"
 #include "frog/mbma_mod.h"
 
 using namespace std;
-using namespace Timbl;
+using namespace folia;
 
 const long int LEFT =  6; // left context
 const long int RIGHT = 6; // right context
@@ -89,7 +89,7 @@ void Mbma::fillMaps() {
   iNames['g'] = "imperative";
   iNames['a'] = "subjunctive";
 }
-  
+
 // BJ: dirty hack with fixed everything to read in tag correspondences
 void Mbma::init_cgn( const string& dir ) {
   string line;
@@ -122,7 +122,7 @@ void Mbma::init_cgn( const string& dir ) {
     throw ( runtime_error( "unable to open:" + fn ) );
 }
   
-void Mbma::init( const Configuration& config ) {
+bool Mbma::init( const Configuration& config ) {
   *Log(theErrLog) << "Initiating morphological analyzer...\n";
   debugFlag = tpDebug;
   string db = config.lookUp( "debug", "mbma" );
@@ -139,9 +139,8 @@ void Mbma::init( const Configuration& config ) {
   if ( opts.empty() )
     opts = "-a1";
   opts += " +vs -vf"; // make Timbl run quietly
-  MTree = new TimblAPI(opts);
-  MTree->GetInstanceBase(MTreeFilename);
-  return;
+  MTree = new Timbl::TimblAPI(opts);
+  return MTree->GetInstanceBase(MTreeFilename);
 }
   
 void Mbma::cleanUp(){
@@ -155,11 +154,11 @@ vector<string> Mbma::make_instances( const UnicodeString& word ){
   if (debugFlag > 2)
     cout << "word: " << word << "\twl : " << word.length() << endl;
   for( long i=0; i < word.length(); ++i ) {
-    if (debugFlag > 2)
+    if (debugFlag > 10)
       cout << "itt #:" << i << endl;
     UnicodeString inst;
     for ( long j=i ; j <= i + RIGHT + LEFT; ++j ) {
-      if (debugFlag > 2)
+      if (debugFlag > 10)
 	cout << " " << j-LEFT << ": ";
       if ( j < LEFT || j >= word.length()+LEFT )
 	inst += "_,";
@@ -170,7 +169,7 @@ vector<string> Mbma::make_instances( const UnicodeString& word ){
 	  inst += word[j-LEFT];
 	inst += ",";
       }
-      if (debugFlag > 2)
+      if (debugFlag > 10)
 	cout << " : " << inst << endl;
     }
     inst += "?";
@@ -407,7 +406,7 @@ void Mbma::resolve_inflections( vector<waStruct>& ana,
 
 MBMAana Mbma::addInflect( const vector<waStruct>& ana,
 			  const string& inflect, 
-			  const string& morph ){
+			  const vector<string>& morph ){
   bool found = false;
   string thisclass;
   vector<waStruct>::const_reverse_iterator it = ana.rbegin();
@@ -449,11 +448,11 @@ MBMAana Mbma::addInflect( const vector<waStruct>& ana,
 
 MBMAana Mbma::inflectAndAffix( const vector<waStruct>& ana ){
   string inflect;
-  string morphemes;
+  vector<string> morphemes;
   vector<waStruct>::const_iterator it = ana.begin();
   while ( it != ana.end() ) { 
     if ( !it->word.isEmpty() ){
-      morphemes += "[" + UnicodeToUTF8(it->word) + "]";
+      morphemes.push_back( UnicodeToUTF8(it->word) );
     }
     string this_class= it->act;
     if (debugFlag)
@@ -511,7 +510,7 @@ MBMAana Mbma::inflectAndAffix( const vector<waStruct>& ana ){
 
 void Mbma::execute( const UnicodeString& word, 
 		    const vector<string>& classes ){
-  string basictags = "NAQVDOBPYIXZ";
+  const string basictags = "NAQVDOBPYIXZ";
   analysis.clear();
   /* determine the largest number of alternative analyses 
      and store every part in a vector of string vectors
@@ -562,9 +561,35 @@ void Mbma::execute( const UnicodeString& word,
     analysis.push_back( tmp );
   }
 }
+
+void Mbma::addMorph( FoliaElement *word, 
+		     const vector<string>& lemmas ){
+  FoliaElement *ml = new MorphologyLayer("");
+#pragma omp critical(foliaupdate)
+  {
+    word->append( ml );
+  }
+  int offset = 0;
+  string args = "annotator='mbma'";
+  for ( size_t p=0; p < lemmas.size(); ++p ){
+    FoliaElement *m = new Morpheme("");
+#pragma omp critical(foliaupdate)
+    {
+      ml->append( m );
+    }
+    FoliaElement *t = 
+      new TextContent( "value='" + escape( lemmas[p]) + 
+			      "', offset='" + toString(offset) + "'" );
+
+    offset += lemmas[p].length();
+#pragma omp critical(foliaupdate)
+    {
+      m->append( t );
+    }
+  }
+}	  
       
-string Mbma::postprocess( const UnicodeString& word,
-			  const string& inputTag ){
+void Mbma::postprocess( FoliaElement *fword ){
   if (debugFlag){
     for(vector<MBMAana>::const_iterator it=analysis.begin(); it != analysis.end(); it++)
       cout << it->getTag() << it->getInflection()<< " ";
@@ -574,21 +599,11 @@ string Mbma::postprocess( const UnicodeString& word,
     cout << endl <<endl;
   }
   
-  string res;
   if (debugFlag)
-      cout << "before morpho: " << res << endl;
-  
-  // BJ: OK, this was taglemma, now get the morphological analysis in
-  string::size_type openH = inputTag.find( '(' );
-  string::size_type closeH = inputTag.find( ')' );
-  if ( openH == string::npos || closeH == string::npos ){
-    *Log(theErrLog) << "main tag without subparts: impossible" << endl;
-    exit(-1);
-  }
-  string mainTag = inputTag.substr( 0, openH );
-  string tagPartS = inputTag.substr( openH+1, closeH-openH-1 );    
-  
-  map<string,string>::const_iterator tagIt = TAGconv.find( mainTag );
+      cout << "before morpho: " << endl;
+  PosAnnotation * pos = fword->annotation<PosAnnotation>();
+  const string tag = pos->feat("head");
+  map<string,string>::const_iterator tagIt = TAGconv.find( tag );
   if ( tagIt == TAGconv.end() ) {
     //
     // this should never happen
@@ -596,6 +611,7 @@ string Mbma::postprocess( const UnicodeString& word,
     if (debugFlag){
       cout << "no match!\n";
     } 
+    throw ValueError( "unknown pos tag value '" + tag + "'" );
   }
   else {
     if (debugFlag){
@@ -621,22 +637,23 @@ string Mbma::postprocess( const UnicodeString& word,
     }
     //should be(come?) a switch
     if ( match == 0 ) {
-      // fallback option: put the word in bracket's and pretend it's a lemma ;-)
-      res = "[" + UnicodeToUTF8(word) + "]";
+      // fallback option: use the word and pretend it's a lemma ;-)
+      vector<string> tmp;
+      UnicodeString word = fword->text();
+      word.toLower();
+      tmp.push_back( UnicodeToUTF8(word) );
+      addMorph( fword, tmp );
     }
     else if (match == 1) {
-      res = analysis[matches[0]].getMorph();
+      const vector<string> ma = analysis[matches[0]].getMorph();
+      addMorph( fword, ma );
     } 
     else {
-      // find the best match
-      vector<string> tagParts;
-      // we need the separate tags from the inputTag
-      
-      size_t numParts = split_at( tagPartS, tagParts, "," );
+      vector<Feature*> feats = fword->annotation<PosAnnotation>()->select<Feature>();
       if (debugFlag){
-	cout << "tag: " << inputTag << ": " << mainTag << endl;
-	for ( size_t q =0 ; q < tagParts.size(); ++q ) {
-	  cout << "\tpart #" << q << ": " << tagParts[q] << endl;
+	cout << "tag: " << tag << endl;
+	for ( size_t q =0 ; q < feats.size(); ++q ) {
+	  cout << "\tpart #" << q << ": " << feats[q]->cls() << endl;
 	}
       }
       if (debugFlag)
@@ -644,33 +661,38 @@ string Mbma::postprocess( const UnicodeString& word,
       map<string, int> possible_lemmas;
       // find unique lemma's
       for ( size_t q = 0; q < match; ++q ) {
-	map<string, int>::iterator lemmaIt
-	  = possible_lemmas.find( analysis[matches[q]].getMorph());
-	if (lemmaIt == possible_lemmas.end())
-	  possible_lemmas.insert( make_pair( analysis[matches[q]].getMorph(),1 ) );
-	else
+	const vector<string> ma = analysis[matches[q]].getMorph();
+	string key;
+	for ( size_t p=0; p < ma.size(); ++p )
+	  key += ma[p];
+	map<string, int>::iterator lemmaIt = possible_lemmas.find( key );
+	if (lemmaIt == possible_lemmas.end()){
+	  possible_lemmas.insert( make_pair( key,1 ) );
+	}
+	else {
 	  lemmaIt->second++;
+	}
       }
       if (debugFlag)
 	cout << "#unique lemma's: " << possible_lemmas.size() << endl;
       if ( possible_lemmas.size() < 1) {
 	if (debugFlag)
 	  cout << "Problem!, no possible lemma's" << endl;
-	return res;
+	return;
       }
       // append all unique lemmas
       // find best match
       // loop through all subParts of the tag
       // and match with inflections from each m
-      set<string> bestMatches;
+      map <string, const vector<string>* > bestMatches;
       int max_count = 0;
       for ( size_t q=0; q < match; ++q ) {
 	int match_count = 0;
 	string inflection = analysis[matches[q]].getInflection();
 	if (debugFlag)
-	  cout << "matching " << inflection << " with " << inputTag << endl;
-	for ( size_t u=0; u < numParts; ++u ) {
-	  map<string,string>::const_iterator conv_tag_p = TAGconv.find(tagParts[u]);
+	  cout << "matching " << inflection << " with " << tag << endl;
+	for ( size_t u=0; u < feats.size(); ++u ) {
+	  map<string,string>::const_iterator conv_tag_p = TAGconv.find(feats[u]->cls());
 	  if (conv_tag_p != TAGconv.end()) {
 	    string c = conv_tag_p->second;
 	    if ( inflection.find( c ) != string::npos )
@@ -678,34 +700,47 @@ string Mbma::postprocess( const UnicodeString& word,
 	  }
 	}
 	if (debugFlag)
-	  cout << "score: " << match_count << endl;
+	  cout << "score: " << match_count << " max was " << max_count << endl;
 	if (match_count >= max_count) {
+	  string key;
+	  const vector<string> *ma = &analysis[matches[q]].getMorph();
+	  for ( size_t p=0; p < ma->size(); ++p )
+	    key += (*ma)[p] + "+"; // create uniqe keys
+	  
 	  if (match_count > max_count) {
 	    max_count = match_count;
 	    bestMatches.clear();
-	    bestMatches.insert( analysis[matches[q]].getMorph() );
-	  }
-	  else {
-	    bestMatches.insert( analysis[matches[q]].getMorph() );
 	  }
+	  bestMatches.insert( make_pair(key, ma ) );
 	}
       }
-      set<string>::const_iterator it = bestMatches.begin();
+      map< string, const vector<string> *>::const_iterator it = bestMatches.begin();
       while ( it != bestMatches.end() ){
-	res += *it;
+	if (debugFlag){
+	  string tmp;
+	  for ( size_t p=0; p < it->second->size(); ++p )
+	    tmp += "[" + (*it->second)[p] + "]";
+	  cout << "MBMA add lemma: " << tmp << endl;
+	}
+	addMorph( fword, *it->second );
 	++it;
-	if ( it != bestMatches.end() )
-	  res += "/";
       }
     }
   }
-  if (debugFlag)
-    cout << "final MBMA lemma: " << res << endl;
-  return res;  
 }  // postprocess
-  
-void Mbma::Classify( const UnicodeString& word ) {
-  UnicodeString uWord = word;
+
+bool Mbma::Classify( FoliaElement* sword ){
+  UnicodeString uWord = sword->text();
+  PosAnnotation * pos = sword->annotation<PosAnnotation>();
+  string tag = pos->feat("head");
+  if ( tag == "SPEC" ){
+    string word = UnicodeToUTF8( uWord );
+    vector<string> tmp;
+    tmp.push_back( word );
+    addMorph( sword, tmp );
+    return true;
+  }
+  uWord.toLower();
   if (debugFlag)
     cout << "Classify word: " << uWord << endl;
   
@@ -713,7 +748,12 @@ void Mbma::Classify( const UnicodeString& word ) {
   vector<string> classes;
   for( size_t i=0; i < insts.size(); ++i ) {
     string ans;
-    MTree->Classify( insts[i], ans);
+    string db;
+    double d;
+    MTree->Classify( insts[i], ans, db, d );
+    if ( debugFlag )
+      cerr << "itt #" << i << ": timbl gave class= " << ans 
+	   << " from distrib= " << db << endl; 
     classes.push_back( ans);
   }
   // fix for 1st char class ==0
@@ -721,6 +761,8 @@ void Mbma::Classify( const UnicodeString& word ) {
     classes[0] = "X";
   
   execute( uWord, classes );
+  postprocess( sword );
+  return true;
 }
 
 ostream& operator<< ( ostream& os, const MBMAana& a ){
